import java.util.HashMap;

public class Analyzer {
	static public void main(String[] args) {
		HashMap<String,Integer> options = new HashMap<String,Integer>();
		String opt="foo";
		// defaults
		options.put("n", 100000000);
		options.put("b", 14);
		options.put("w", 6);
		options.put("d", 0);
		options.put("o", 10);
		for (int i=0; i<args.length; i++) {
			if (args[i].startsWith("-")) {
				opt = args[i].substring(1);
				options.put(opt,0);
			} else {
				options.put(opt,Integer.parseInt(args[i]));
			}
		}
		if (options.containsKey("h") || options.containsKey("help")) {
			usage();
			System.exit(0);
		}
		
		int point=0, passOdds=0, dontOdds=0, games=0, value, count, rolls;
		double net=0;
		int maxOdds = options.get("o");
		boolean passline = options.containsKey("p");
		boolean comeout=true;
		Window window = new Window(options.get("w"));
		PointStats stats = new PointStats(options.get("w"));
		boolean analyze = options.containsKey("a");
		CSM csm = new CSM();
		Roll roll;
		csm.setMinBufferDepth(options.get("b"));
		System.out.println(String.format("%s, %dx odds, %d card min buffer depth, %d-roll window",
			(passline ? "pass line" : "don't pass"), maxOdds, options.get("b"), options.get("w")));
		rolls = options.get("n");
		for (int i=0; i<rolls; i++) {
			roll = csm.dealRoll();
			value = roll.total();
			if (comeout) games++;
			if (!comeout) {
				if (value == point) {
					if (passline) {
						net += (1.0 + passOdds*pointOdds(point));
					} else {
						net -= (1.0 + dontOdds*pointOdds(point));
					}
					comeout = true;
				} else if (value == 7) {
					if (passline) {
						net -= (1.0 + passOdds);
					} else {
						net += (1.0 + dontOdds);
					}
					comeout = true;
				}
			} else if ((value == 7) || (value == 11)) {
				net += passline ? 1 : -1;
			} else if ((value == 2) || (value == 3)) {
				net += passline ? -1 : 1;
			} else if (value == 12) {
				// boxcars
				if (passline) net -= 1.0;
			} else {
				point = value;
				comeout = false;
			}
			if (analyze) stats.addRoll(roll);
			window.push(roll);
			if (!comeout) {
				count = window.getCount(point);
				switch (point) {
					case 4: case 10:
					case 6: case 8:
						dontOdds = (count >= 0) ? maxOdds : 0;
						passOdds = (count < 0)  ? maxOdds : 0;
						break;
					case 5: case 9:
						dontOdds = (count > 0) ? maxOdds : 0;
						passOdds = (count < 0) ? maxOdds : 0;
						break;
				}
			}
			if (i%1000000 == 0) {
				System.out.println(String.format("%d rolls: net %+.1f, EV/roll %+4.2f%%, EV/game %+4.2f%%",
					i, net, 100.0*net/(double)i, 100.0*net/(double)games));
			}
			csm.shuffleRoll(roll);
		}
		System.out.println(String.format("%d rolls: net %+.1f, EV/roll %+4.2f%%, EV/game %+4.2f%%",
			rolls, net, 100.0*net/(double)rolls, 100.0*net/(double)games));
		if (analyze) stats.printSummary();
	}
	
	static double pointOdds(int point) {
        switch (point) {
            case 4: case 10:
                return 2;
            case 5: case 9:
                return 1.5;
            case 6: case 8:
                return 1.2;
            default:
                return 0;
        }
	}
	
	static public void usage() {
		System.out.println("usage: Analyzer <options>");
		System.out.println("where options include:");
		System.out.println("  -n <number of rolls>             specifies number of rolls to simulate (default 100 million)");
		System.out.println("  -d                               play Don't Pass line (default)");
		System.out.println("  -p                               play Pass line");
		System.out.println("  -o <max odds>                    specifies odds to take/lay for good count");
		System.out.println("  -b <buffer depth>                specifies minimum reservoir depth (default 14)");
		System.out.println("  -w <window depth>                specifies count window depth (default 6)");
		System.out.println("  -a                               print per-point statistics");
		System.out.println("  -h, --help                       display this usage");
		System.out.println("Place a space between the option and parameter value.");
	}

}