import java.util.Vector;
import java.util.Random;

/**
 *
 * @author show
 */
public class CSM {
    // params approximate the ShuffleMaster 126 model Constant Shuffle Machine
    static final int NUM_SLOTS = 40;
    static final int DICE_SETS = 52;
    protected int min_buffer_depth = 12;  // minimum cards in chute
    protected Vector<Integer>[] slot = new Vector[NUM_SLOTS];    // card slots in "wheel"
    protected Vector<Integer> chute = new Vector<Integer>();  // cards are dealt from chute
    protected Random random = new Random();

    public CSM() {
        for (int i=0; i<NUM_SLOTS; i++) {
            slot[i] = new Vector<Integer>();
        }
        // load dice
        for (int i=0; i<DICE_SETS; i++) {
            for (int d=1; d<=6; d++) {
                shuffleCard(d);
            }
        }
    }

    // shuffle roll back into CSM
    public void shuffleRoll(Roll roll) {
        shuffleCard(roll.getDie1());
        shuffleCard(roll.getDie2());
    }

    public Roll dealRoll() {
        return new Roll(dealCard(), dealCard());
    }

	public void setMinBufferDepth(int depth) {
		min_buffer_depth = depth;
	}

    // deal card from chute
    protected int dealCard() {
        // drop a slot if chute is running out of cards
        while (chute.size() < min_buffer_depth) {
            // select a random slot of cards from the "wheel"
            Vector<Integer> randomSlot = slot[random.nextInt(NUM_SLOTS)];
            // drop slot of cards into the chute
            chute.addAll(randomSlot);
            randomSlot.clear();
        }
        return chute.remove(0);
    }

    // shuffle card into random position of random slot
    protected void shuffleCard(int card) {
        Vector<Integer> randomSlot = slot[random.nextInt(NUM_SLOTS)];
        if (randomSlot.isEmpty()) {
            randomSlot.add(card);
        } else {
            int randomPosition = random.nextInt(randomSlot.size());
            randomSlot.add(randomPosition, card);
        }
    }
}
